(*
 * Copyright 2020 Arne Vajhøj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 *)

module sqlite;

[external('decc$crtl_init')]
procedure crtl_init; extern;
[external('decc$free')]
procedure free_c_str(ptr : c_str_t); extern;

type
   sqlite_ptr = integer_address;
   sqlite_stmt_ptr = integer_address;

(* sqlite3.h: *)

const
   SQLITE_OK          = 0;
   SQLITE_ERROR       = 1;
   SQLITE_INTERNAL    = 2;
   SQLITE_PERM        = 3;
   SQLITE_ABORT       = 4;
   SQLITE_BUSY        = 5;
   SQLITE_LOCKED      = 6;
   SQLITE_NOMEM       = 7;
   SQLITE_READONLY    = 8;
   SQLITE_INTERRUPT   = 9;
   SQLITE_IOERR       = 10;
   SQLITE_CORRUPT     = 11;
   SQLITE_NOTFOUND    = 12;
   SQLITE_FULL        = 13;
   SQLITE_CANTOPEN    = 14;
   SQLITE_PROTOCOL    = 15;
   SQLITE_EMPTY       = 16;
   SQLITE_SCHEMA      = 17;
   SQLITE_TOOBIG      = 18;
   SQLITE_CONSTRAINT  = 19;
   SQLITE_MISMATCH    = 20;
   SQLITE_MISUSE      = 21;
   SQLITE_NOLFS       = 22;
   SQLITE_AUTH        = 23;
   SQLITE_FORMAT      = 24;
   SQLITE_RANGE       = 25;
   SQLITE_NOTADB      = 26;
   SQLITE_NOTICE      = 27;
   SQLITE_WARNING     = 28;
   SQLITE_ROW         = 100;
   SQLITE_DONE        = 101;

[external('sqlite3_open')]
function sqlite3_open(%IMMED db : c_str_t; %REF con : sqlite_ptr) : integer; extern;

[external('sqlite3_prepare')]
function sqlite3_prepare(%IMMED con : sqlite_ptr; %IMMED sqlstr : c_str_t; %IMMED sqllen : integer; %REF stmt : sqlite_stmt_ptr; %IMMED remain : integer := 0) : integer; extern;

[external('sqlite3_bind_int')]
function sqlite3_bind_int(%IMMED stmt : sqlite_stmt_ptr; %IMMED ix : integer; %IMMED ival : integer) : integer; extern;

[external('sqlite3_bind_double')]
function sqlite3_bind_double(%IMMED stmt : sqlite_stmt_ptr; %IMMED ix : integer; %IMMED xval : double) : integer; extern;
                                                        
[external('sqlite3_bind_text')]
function sqlite3_bind_text(%IMMED stmt : sqlite_stmt_ptr; %IMMED ix : integer; %IMMED sval : c_str_t; %IMMED slen : integer; %IMMED f : integer := 0) : integer; extern;

[external('sqlite3_bind_null')]
function sqlite3_bind_null(%IMMED stmt : sqlite_stmt_ptr; %IMMED ix : integer) : integer; extern;

[external('sqlite3_step')]
function sqlite3_step(%IMMED stmt : sqlite_stmt_ptr) : integer; extern;

[external('sqlite3_column_int')]
function sqlite3_column_int(%IMMED stmt : sqlite_stmt_ptr; %IMMED ix : integer) : integer; extern;

[external('sqlite3_column_double')]
function sqlite3_column_double(%IMMED stmt : sqlite_stmt_ptr; %IMMED ix : integer) : double; extern;

[external('sqlite3_column_text')]
function sqlite3_column_text(%IMMED stmt : sqlite_stmt_ptr; %IMMED ix : integer) : c_str_t; extern;

[external('sqlite3_finalize')]
procedure sqlite3_finalize(%IMMED stmt : sqlite_stmt_ptr); extern;

[external('sqlite3_close')]
procedure sqlite3_close(%IMMED con : sqlite_ptr); extern;

[external('sqlite3_errmsg')]
function sqlite3_errmsg(%IMMED con : sqlite_ptr) : c_str_t; extern;

end.
